#include <sys/types.h>
#include <gtk/gtk.h>
#include <gtkextra/gtkextra.h>
#include <Python.h>
#include <grammar.h>
#include <node.h>
#include <parsetok.h>
#include <errcode.h>
#include <compile.h>
#include <eval.h>
#include <marshal.h>
#ifdef WITH_NUMERIC_PYTHON
#include <arrayobject.h>
#endif
#include "../main.h"
#include "python_int.h"

extern PyObject *main_o, *main_dict, *sg_o, *sg_dict;

extern gint stdout_pipe[2];
gint py_status=0; /* 0=normal, 1=old line waiting */
extern grammar _PyParser_Grammar; /* From Python/graminit.c */
extern FILE *stdout_fp;
extern pid_t child_pid;
GString *com_buffer;

#ifndef Py_eval_input
#include <graminit.h>
#define Py_eval_input eval_input
#endif /* Py_eval_input */

int sg_eval_func(gchar *func_def, gdouble x_value, gdouble *y_value)
{ PyObject *object=NULL;
  gint len;

  object=Py_BuildValue("d",x_value);
  if (!object) return FALSE;
  PyDict_SetItemString (sg_dict, "x", object);

  object=PyRun_String (func_def, Py_eval_input, sg_dict, sg_dict);
  if (!object)
   { if (PyErr_Occurred()) python_error_report(object);
     return FALSE;
   }

  *y_value=PyFloat_AsDouble (object);

  return TRUE;
}

/* Evalutes an expression and sends back an array of doubles */
gdouble *sg_eval_expr_double(gchar *expr, gint *num, gint expect)
{ PyObject *object=NULL,*seq_obj;
  PyArrayObject *parray;
  gpointer apt;
  GArray *array;
  double *retarray,val;
  gint i,j,len;

  *num=0;

  array=g_array_new (FALSE, FALSE, sizeof (gdouble));
  object=PyRun_String (expr, Py_eval_input, main_dict, sg_dict);
  if (!object)
   { if (PyErr_Occurred()) { PyErr_Print(); PyErr_Clear();}
     *num=0;
   }

#ifdef WITH_NUMERIC_PYTHON
  else {
  if (PyArray_Check(object) && ((PyArrayObject *)object)->nd==1)
  {    parray=(PyArrayObject *)object;
       for (i=0;i<parray->dimensions[0];i++)
          { apt=parray->data+i*parray->strides[0];
            switch (parray->descr->type_num)
             { case PyArray_CHAR:
               case PyArray_UBYTE:
               case PyArray_SBYTE:
                val=(double)(*(char *)(apt));
                g_array_append_val( array,val);
                break;
               case PyArray_SHORT:
                val=(double)(*(short int *)(apt));
                g_array_append_val( array,val);
                break;
               case PyArray_INT:
                val=(double)(*(int *)(apt));
                g_array_append_val( array,val);
                break;
               case PyArray_LONG:
                val=(double)(*(long *)(apt));
                g_array_append_val( array,val);
                break;
               case PyArray_FLOAT:
                val=(double)(*(float *)(apt));
                g_array_append_val( array,val);
                break;
               case PyArray_DOUBLE:
                val=*(double *)(apt);
                g_array_append_val( array,val);
                break;
               case PyArray_CFLOAT:
               case PyArray_CDOUBLE:
                val=(double)(*(long double*)(apt));
                g_array_append_val( array,val);
                break;
               case PyArray_OBJECT:
                break;
             }
          }
  }
  else
#endif
  if (PySequence_Check(object) && !PyString_Check(object))
  {   len=PySequence_Length(object);
      for (i=0;i<len;i++)
       { seq_obj=PySequence_GetItem(object,i);
         Py_INCREF(seq_obj);
         if (PyFloat_Check (seq_obj))
         {
             val=PyFloat_AsDouble(seq_obj);
             g_array_append_val( array, val);
         }
         else if (PyInt_Check (seq_obj))
         {
             val=(double)PyInt_AsLong(seq_obj);
             g_array_append_val( array, val);
         }
       }
  }
  else if (PyFloat_Check (object))
  {
      val=PyFloat_AsDouble(object);
      g_array_append_val( array, val);
  }
  else if (PyInt_Check (object))
  {
      val=(double)PyInt_AsLong(object);
      g_array_append_val( array, val);
  }
 } 
 len=expect-array->len;
 if (expect>0 && array->len<expect)
    for (i=0;i<len;i++)
    { val=0;
      g_array_append_val(array,val);
    }
 retarray=(double *)(array->data);
 *num=array->len;
 g_array_free(array,FALSE); /* Delete structure, but NOT the data */

 return retarray;
}

/* Evalutes an expression and sends back an array of strings */
gchar **sg_eval_expr_string(gchar *expr, gint *num)
{ PyObject *object=NULL,*seq_obj;
  GArray *array;
  gchar **retarray,*val;
  gint i,j,len;

  *num=0;

  object=PyRun_String (expr, Py_eval_input, main_dict, sg_dict);
  if (!object)
   { if (PyErr_Occurred()) { PyErr_Print(); PyErr_Clear();}
     return NULL;
   }

  array=g_array_new (FALSE, FALSE, sizeof (gchar *));
  if (PySequence_Check(object) && !PyString_Check(object))
  {   len=PySequence_Length(object);
      for (i=0;i<len;i++)
       { seq_obj=PySequence_GetItem(object,i);
         Py_INCREF(seq_obj);
         if (PyString_Check (seq_obj))
         {
             val=strdup(PyString_AsString(seq_obj));
             g_array_append_val( array,  val);
         }
         else
         {
             val=strdup(PyString_AsString( PyObject_Str(seq_obj)));
             g_array_append_val( array, val);
         }
       }
  }
  else if (PyString_Check (object))
  {
      val=strdup(PyString_AsString(object));
      g_array_append_val( array, val);
  }
  else
  {
      val=strdup(PyString_AsString( PyObject_Str(object)));
      g_array_append_val( array,  val);
  }

  retarray=(gchar  **)(array->data);
  *num=array->len;
  g_array_free(array,FALSE); /* Delete structure, but NOT the data */

  return retarray;
}



int python_simple(gchar *command,gint count)
{ int retval,len;
  static PyObject *object=NULL;
  gchar *print_string,fn[]=PACKAGE,*s_str;
  node *parse_node;
  perrdetail perr;

  command[count]=0;
  if (py_status==0)
   com_buffer=g_string_new(command);
  else
   g_string_append(com_buffer,command);

  parse_node = PyParser_ParseString(com_buffer->str, &_PyParser_Grammar,
                                    Py_file_input,&perr);
  switch (perr.error)
  { case  E_EOF:
     py_status=1;
     return(2);
    case  E_DONE:
      if (command[0]=='\n')
       { if (parse_node) PyNode_Free(parse_node);
	 py_status=0;
	 g_string_free(com_buffer,TRUE);
	 return(1);
       }
    default:
     py_status=0;
     if (parse_node) PyNode_Free(parse_node);
  }

  object=PyRun_String (com_buffer->str, Py_single_input, main_dict, sg_dict);

  if (!object){
      PyErr_Print();
      if (Py_FlushLine())
          PyErr_Clear(); 
      g_string_free(com_buffer,TRUE);
      return(0); 
  }
   
   
  Py_DECREF(object);
  g_string_free(com_buffer,TRUE);
 return(1);
}
