/* This is the interface to the python interpreter
   Originally from the now defunct gnocci project.
   C. Steenberg 15 May 2000
*/

#include <gdk/gdk.h>
#include <glib.h>
#include <gtkextra/gtkextra.h>
#include <gdk/gdkkeysyms.h>
#include <stdio.h>
#include <string.h>
#include <Python.h>
#include <grammar.h>
#include <node.h>
#include <parsetok.h>
#include <errcode.h>
#include <compile.h>
#include <eval.h>
#include <marshal.h>
#ifdef WITH_NUMERIC_PYTHON
#include <arrayobject.h>
#endif
#include "../main.h"
#include "python_int.h"

gchar def_safe_modules[][80]={"audioop", "array","binascii", "cmath","errno", 
      "imageop","marshal", "math", "md5", "operator","parser", "regex", "rotor", 
      "select","strop", "struct","time",""};
gchar def_safe_os[][80]={"error", "fstat","listdir", "lstat", "readlink",
      "stat","times", "uname", "getpid", "getppid","getcwd","getuid", "getgid",
      "geteuid","getegid","name","environ","ctermid","getgroups","getlogin",
      "getpgrp","getpid","getppid","strerror","umask","uname","fpathconf","isatty",
      "times","confstr","confstr_names","sysconf","sysconf_names","curdir",
      "pardir","sep","altsep","pathsep","defpath","linesep","path",""};
gchar def_safe_sys[][80]={"ps1","ps2", "copyright", "version", "platform","exit",
      "maxint","stderr","stdout","stdin","byteorder","builtin_module_names",
      "executable","exitfunc","getrefcount","getrecursionlimit","maxint","platform",
      "prefix","version_info","path",""};

module_opts comp_imports[]=
{
 {"Numeric",SG_MODULE_LOAD_SYMBOLS|SG_MODULE_LOAD_REQ},
 {"pickle",SG_MODULE_LOAD_MAIN|SG_MODULE_LOAD_REQ},  
  {NULL,0}};

extern grammar _PyParser_Grammar; /* From Python/graminit.c */
extern pid_t child_pid;

static PyObject *sg_import(PyObject *self, PyObject *args);
gint python_error_report(PyObject *object);

/* Remember to increase the size of save_vars and save_obj when adding to this list*/
gchar *save_vars_names[]={"safe_modules","safe_os","safe_sys","import_modules",""};


gint python_init(gchar *progname)
{   PyObject *module_object, *module_dict, *keys, *functions, *key_name, *object,
             *full, *empty, *items, *string;
    gint i,flag;
    gchar temp[200],*name,*homedir; 
    Py_SetProgramName(progname);
    Py_Initialize();

    sg_report_python_error=TRUE;
    for (i=0;strlen(save_vars_names[i])>0;i++)
      { save_vars[i]=save_vars_names[i];
        save_obj[i]=NULL;
      }
    save_vars[i]=save_vars_names[i]; /* The sentinel */
    SG_IMPORT_MODULES=PyDict_New();
    
    main_o=PyImport_ImportModule("__main__");
    builtin_o=PyImport_AddModule("__builtins__");
    sys_o=PyImport_ImportModule ("sys");
    sys_o=PyImport_AddModule ("sys");    
    os_o=PyImport_ImportModule ("os");
    os_o=PyImport_AddModule ("os");    

    main_dict=PyModule_GetDict(main_o);
    sys_dict=PyModule_GetDict(sys_o);
    os_dict=PyModule_GetDict(os_o);
    config_dict=PyDict_New();    
    builtin_dict=PyModule_GetDict(builtin_o);

    PyDict_SetItemString (main_dict, "os", os_o);
    PyDict_SetItemString (main_dict, "sys", sys_o);


/* sg module init */
    init_sg_python();
    sg_o=PyImport_AddModule("sg");
    sg_dict=PyModule_GetDict(sg_o);
    PyDict_SetItemString (main_dict, "sg", sg_o);  
    PyDict_SetItemString (main_dict, "__import__", 
    PyDict_GetItemString (sg_dict, "__import__"));


/* Import compulsory modules, and put their names in the main dict */
    full=PyList_New (1);    
    string=PyString_FromString ("*");
    PyList_SET_ITEM (full, 0, string);
    Py_XDECREF(string);
    
    for (i=0;comp_imports[i].name!=NULL;i++)
    { module_object=PyImport_ImportModuleEx (comp_imports[i].name, main_dict, sg_dict, full);
      if (!module_object) continue;
      if (comp_imports[i].flags&SG_MODULE_LOAD_SYMBOLS)
        { snprintf(temp,200,"from %s import *",comp_imports[i].name);
          object=PyRun_String (temp, Py_file_input, main_dict, sg_dict);
          python_error_report(object);
        }
      else if (comp_imports[i].flags&SG_MODULE_LOAD_MAIN)
        PyDict_SetItemString (main_dict, comp_imports[i].name, module_object);  
      PyDict_SetItemString(SG_IMPORT_MODULES,comp_imports[i].name,
       PyInt_FromLong ((long) comp_imports[i].flags));
    }

   homedir=getenv("HOME");
   rcfile=g_new0(gchar,strlen(homedir)+6);
   sprintf(rcfile,"%s/.sga",homedir);
   flag=unpickle_file(rcfile, main_dict, sg_dict, save_vars);
   if (flag==TRUE)
     for (i=0;save_vars[i]!=NULL;i++)
        save_obj[i]=PyDict_GetItemString (sg_dict, save_vars[i]);
        
   else /* Set up default values when rc file read fails */
   {    /* SG_IMPORT_MODULES is an exception, it is already set up above */
/* Set up the safe modules list */
     SG_SAFE_MODULES=PyDict_New();
      for (i=0;strlen(def_safe_sys[i])>0;i++)
        PyDict_SetItemString(SG_SAFE_MODULES,def_safe_modules[i],Py_None);

/* Set up the safe sys methods list */
     SG_SAFE_SYS=PyDict_New();    
     for (i=0;strlen(def_safe_sys[i])>0;i++)
          PyDict_SetItemString(SG_SAFE_SYS, def_safe_sys[i],Py_None);

/* Set up the safe os methods list */
     SG_SAFE_OS=PyDict_New();
     for (i=0;strlen(def_safe_os[i])>0;i++)
          PyDict_SetItemString(SG_SAFE_OS, def_safe_os[i],Py_None);

/* Put the object references into the local dictionary */
     for (i=0;save_obj[i];i++)
      PyDict_SetItemString (sg_dict, save_vars[i], save_obj[i]);
   }

   sg_load_default_modules();

/* Now remove naughty functions from sys and os modules */
   def_sys_dict= PyDict_New();
   keys=PyDict_Keys(sys_dict);

   for (i=0;i<PyList_GET_SIZE (keys);i++)
     { key_name=PyList_GetItem (keys,i);
       PyDict_SetItem (def_sys_dict, key_name, PyDict_GetItem (sys_dict, key_name));
       if (!PyDict_GetItem(SG_SAFE_SYS,key_name))
         PyDict_DelItem (sys_dict, key_name);
     }
   Py_XDECREF(keys);
   def_os_dict= PyDict_New();
   keys=PyDict_Keys(os_dict);

   for (i=0;i<PyList_GET_SIZE (keys);i++)
     { key_name=PyList_GetItem (keys,i);
       PyDict_SetItem (def_os_dict, key_name, PyDict_GetItem (os_dict, key_name));
       if (!PyDict_GetItem(SG_SAFE_OS,key_name))
         PyDict_DelItem (os_dict, key_name);
     }
   Py_XDECREF(keys);
   pickle_file(rcfile, main_dict, sg_dict, save_vars);
   return(1);
}

/* Code for custom module imports */
gint sg_load_default_modules(void)
{ PyObject *keys, *items, *key_name, *module_object,*object,*full,*string;
  gint i=0,flag;
  gchar *name,temp[200];
  
    keys=PyDict_Keys(SG_IMPORT_MODULES);
    items=PyDict_Items(SG_IMPORT_MODULES);

    full=PyList_New (1);    
    string=PyString_FromString ("*");
    PyList_SET_ITEM (full, 0, string);
    
    for (i=0;i<PyList_GET_SIZE (keys);i++)
    { key_name=PyList_GetItem (keys,i);
      name=PyString_AsString (key_name);
      flag=(gint)PyInt_AsLong (PyList_GetItem (items,i));
      module_object=PyImport_ImportModuleEx (name, main_dict, sg_dict, full);
      if (!module_object) continue;
      if (flag&SG_MODULE_LOAD_SYMBOLS)
        { snprintf(temp,200,"from %s import *",name);
          object=PyRun_String (temp, Py_file_input, main_dict, sg_dict);
          python_error_report(object);
          Py_XDECREF(object);
        }
      else if (flag&SG_MODULE_LOAD_MAIN)
        PyDict_SetItemString (main_dict, name, module_object);  
    }

    Py_XDECREF(full);
    Py_XDECREF(string);
    Py_XDECREF(keys);
    Py_XDECREF(items);
    return i;
}

gint pickle_file(gchar *fname, PyObject *globals, PyObject *locals, gchar **symbols)
{ gint i=0;
  FILE *fp;
  PyObject *object, *fo;
  gchar temp[200];

  if (!(fp=fopen(fname,"w"))) return FALSE;
  fo=PyFile_FromFile (fp, "pickle_file", "w", NULL);
  PyDict_SetItemString (sg_dict, "pickle_file", fo);
  snprintf(temp,200,"pickler=pickle.Pickler(pickle_file)");
  object=PyRun_String (temp, Py_file_input, globals, locals);
  if (python_error_report(object))
     return FALSE;

  for (i=0;strlen(symbols[i])>0;i++)
   { snprintf(temp,200,"pickler.dump(%s)",symbols[i]);
     object=PyRun_String (temp, Py_file_input, globals, locals);
     python_error_report(object);
   }
  snprintf(temp,200,"del pickle_file,pickler",symbols[i]);  
  object=PyRun_String (temp, Py_file_input, globals, locals);
  if (python_error_report(object))
   return (FALSE);
  fclose(fp);
  return TRUE; 
}

gint unpickle_file(gchar *fname, PyObject *globals, PyObject *locals, gchar **symbols)
{
  gint i=0;
  FILE *fp;
  PyObject *object, *fo;
  gchar temp[200];

  if (!(fp=fopen(fname,"r"))) return FALSE;
  fo=PyFile_FromFile (fp, "pickle_file", "r", NULL);
  PyDict_SetItemString (sg_dict, "pickle_file", fo);
  snprintf(temp,200,"unpickler=pickle.Unpickler(pickle_file)");
  object=PyRun_String (temp, Py_file_input, globals, locals);
  if (python_error_report(object))
    { Py_XDECREF(object);
      fclose(fp);
      Py_XDECREF(fo);
 	return FALSE;
    }

   for (i=0;strlen(symbols[i])>0;i++)
   { snprintf(temp,200,"%s=unpickler.load()",symbols[i]);
     object=PyRun_String (temp, Py_file_input, globals, locals);
     if (python_error_report(object))
     { Py_XDECREF(object);
       fclose(fp);
       Py_XDECREF(fo);
  	return FALSE;
     }

   }
  snprintf(temp,200,"del pickle_file,unpickler",symbols[i]);  
  object=PyRun_String (temp, Py_file_input, globals, locals);
  python_error_report(object);
  
  fclose(fp);
  Py_XDECREF(object);
  Py_XDECREF(fo);
  return TRUE; 

}

gint python_error_report(PyObject *object)
{ 
    if (!object)
      { if (sg_report_python_error)
          create_python_term(NULL,NULL);
        PyErr_Print();
        if (Py_FlushLine())
            PyErr_Clear(); 
        return TRUE;
      }
    return FALSE;
}
