/*  SciGraphica - Scientific graphics and data manipulation
 *  Copyright (C) 2000 Adrian E. Feiguin <feiguin@ifir.edu.ar>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include <gtk/gtk.h>
#include <gtkextra/gtkextra.h>
#include "sg_worksheet.h"
#include "sg_clipboard.h"

static GList *selection;

void
sg_clipboard_copy(SGworksheet *sg_sheet, gboolean clear)
{
  gint row, col;
  gchar *text, *ftext, *stext;
  GtkSheetCell *cell;
  SGhiddencell *link;
  GtkSheet *sheet=GTK_SHEET(sg_sheet->sheet);
  gint srow,scol,num_selected=0;

  sg_clipboard_clear();

  gtk_sheet_freeze(sheet);
  gtk_sheet_get_active_cell(sheet,&srow,&scol);  

  num_selected=((sheet->range.rowi-sheet->range.row0)+1)*
               ((sheet->range.coli-sheet->range.col0)+1);
  for(row = sheet->range.row0; row <= sheet->range.rowi; row++){
    for(col = sheet->range.col0; col <= sheet->range.coli; col++){
       ftext=NULL;
       stext=NULL;
       text = gtk_sheet_cell_get_text(sheet, row, col);
       link = (SGhiddencell *)gtk_sheet_get_link(sheet, row, col);
       if (link) ftext = link->formula;

       if (row==srow && col==scol && num_selected==1)
        stext=gtk_entry_get_text(GTK_ENTRY(sheet->sheet_entry));

       if (stext && strlen(stext)>0) text=stext;
       else if (ftext && strlen(ftext)>0) text=ftext;
              
       if(text && strlen(text)>0){
           cell = g_new(GtkSheetCell, 1);
           cell->text = g_strdup(text);
           cell->row = row - sheet->range.row0;
           cell->col = col - sheet->range.col0;
          /* This way we tell the paste function we are sending a formula: */
           cell->link = link;
           selection = g_list_append(selection, cell);
       }
    }
  }

  if (clear)
   for (row = sheet->range.row0; row <= sheet->range.rowi; row++)
    for (col = sheet->range.col0; col <= sheet->range.coli; col++)
      { /*gtk_sheet_cell_clear(sheet, row, col); Don't know why this causes segfault*/
        sg_worksheet_cell_set(sg_sheet, row, col, NULL, FALSE,FALSE);
        if (row==srow && col==scol && cell->link)
         gtk_entry_set_text(GTK_ENTRY(sheet->sheet_entry),g_strdup(""));
      }      

  gtk_sheet_thaw(sheet);
}

void
sg_clipboard_paste(SGworksheet *sg_sheet)
{
  GtkSheetCell *cell;
  GList *list;
  GtkSheet *sheet=GTK_SHEET(sg_sheet->sheet);
  gboolean selected;
  gint row,col,srow,scol;

  gtk_sheet_freeze(sheet);

  list = selection;
  gtk_sheet_get_active_cell(sheet,&srow,&scol);
  while(list){
   cell = (GtkSheetCell *)list->data;
   row=sheet->range.row0 + cell->row;
   col=sheet->range.col0 + cell->col;
   if (cell->link) /* We were sent a formula, insert and evaluate */
    sg_worksheet_cell_set(sg_sheet, row, col,
                          cell->text, TRUE, TRUE);
   else /* we were sent the result of a formula elsewhere, just insert as text */
    sg_worksheet_cell_set(sg_sheet, row, col, 
                          cell->text, FALSE, FALSE);
   /* We are pasting into a selected cell, put text in the entry widget */
   if (row==srow && col==scol)
    { if (cell->link)
         gtk_entry_set_text(GTK_ENTRY(sheet->sheet_entry),g_strdup(cell->text));
      else
         gtk_entry_set_text(GTK_ENTRY(sheet->sheet_entry),g_strdup(""));
    }
    list = list->next;
  }
  gtk_sheet_thaw(sheet);
}

void
sg_clipboard_clear()
{
  GtkSheetCell *cell;
  GList *list;

  list = selection;
  while(list){
    cell = (GtkSheetCell *)list->data;
    g_free(cell->text);
    g_free(cell);

    selection = g_list_remove_link(selection, list);
    g_list_free_1(list);
    list = selection;
  }

  sg_clipboard_init();
}

void
sg_clipboard_init()
{
  selection = NULL;
}
