/*  SciGraphica - Scientific graphics and data manipulation
 *  Copyright (C) 2000 Adrian E. Feiguin <feiguin@ifir.edu.ar>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <gtk/gtk.h>
#include <gtkextra/gtkextra.h>
#include "main.h"
#include <Python.h>
#include "arrayobject.h"
#include "python/python_int.h"
#include "python/import_data.h"



static FILE *fp;


void
sg_worksheet_file_import_block(SGworksheet *worksheet, gchar *file)
{
  GtkSheet *sheet;
  GArray *array;
  GPtrArray *titles;
  PyObject *parray;
  gint row, col, x, y, i,temp_line;
  gboolean newcol = TRUE;
  gboolean space = FALSE;
  gchar *line,delim[20]="";
  gchar label[255];

  sheet = GTK_SHEET(worksheet->sheet);
  row = 0;
  col = 0;

  if (use_custom_delim)
   strncat(delim,custom_delimiter,20);
  else
      switch (delimiter)
      {
          case SG_DELIMITER_COMMA:
             strncat(delim,",",20);
             break;

      }

  strncat(delim," \t\n",20);
  if (read_method==0)
  {
      if (!read_all_lines) temp_line=end_line;
      array=read_table(file,comment_string, delim,NULL,0,begin_line,temp_line,&x, &y, &titles);
  }
  else
  {
      array=read_table(file,comment_string, delim,block_start,blocknum,0,0,&x, &y, &titles);
  }
  if (array)
  {
      parray=python_read_build_array(array, x, y);
      Py_INCREF(parray);
  }

  if (!array || !parray)
  {
    g_warning("ERROR: Could not import data from: %s", file);
    return;
  }
  gtk_sheet_get_active_cell(sheet,&row,&col);
  python_insert_object(worksheet, row, col, parray,
                       GTK_ORIENTATION_VERTICAL,TRUE);
  if(titles && titles->len>0 && rename_columns)
      for (i=0;i<titles->len;i++)
      { 
          gtk_sheet_column_button_add_label(sheet, col+i, titles->pdata[i]);
          gtk_sheet_set_column_title(sheet, col+i, titles->pdata[i]);
      }
  Py_XDECREF(parray);
}


gchar *
xml_get_string(gchar *input)
{
  gchar *c;
  gchar *output = NULL;
  gint n = 1;

  output = (gchar *)g_malloc(sizeof(gchar));

  c = input;
  while(c && *c != '\0' && *c != '\n'){
    switch(*c){
        case '>':
          output = (gchar *)g_realloc(output, (n + 3) * sizeof(gchar));
          output [n - 1] = '&';
          output [n] = 'g';
          output [n + 1] = 't';
          output [n + 2] = ';';
          n += 4;
          break;
        case '<':
          output = (gchar *)g_realloc(output, (n + 3) * sizeof(gchar));
          output [n - 1] = '&';
          output [n] = 'l';
          output [n + 1] = 't';
          output [n + 2] = ';';
          n += 4;
          break;
        case '&':
          output = (gchar *)g_realloc(output, (n + 4) * sizeof(gchar));
          output [n - 1] = '&';
          output [n] = 'a';
          output [n + 1] = 'm';
          output [n + 2] = 'p';
          output [n + 3] = ';';
          n += 5;
          break;
        case '\"':
          output = (gchar *)g_realloc(output, (n + 5) * sizeof(gchar));
          output [n - 1] = '&';
          output [n] = 'q';
          output [n + 1] = 'u';
          output [n + 2] = 'o';
          output [n + 3] = 't';
          output [n + 4] = ';';
          n += 6;
          break;
        default:
          output = (gchar *)g_realloc(output, n * sizeof(gchar));
          output[n-1] = *c;
          n += 1;
          break;
    }
    c++;
  }

  output = (gchar *)g_realloc(output, n * sizeof(gchar));
  output[n-1] = '\0';
  return output;
}


