/*  SciGraphica - Scientific graphics and data manipulation
 *  Copyright (C) 2000 Adrian E. Feiguin <feiguin@ifir.edu.ar>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include <math.h>
#include <stdio.h>
#include <gdk/gdk.h>
#include <gtk/gtk.h>
#include <gtkextra/gtkextra.h>
#include "main.h"
#include "sg_text_dialog.h"

static void sg_menu_fit_page				(GtkWidget *widget, 
							 gpointer data);
static void sg_menu_export				(GtkWidget *widget, 
							 gpointer data);
static void sg_menu_import				(GtkWidget *widget, 
							 gpointer data);
static void sg_menu_new_layer				(GtkWidget *widget, 
							 gpointer data);
static void sg_menu_new_layer_3d			(GtkWidget *widget, 
							 gpointer data);
static void sg_menu_new_layer_polar			(GtkWidget *widget, 
							 gpointer data);
static void sg_menu_remove_layer			(GtkWidget *widget, 
							 gpointer data);
static void sg_menu_arrange				(GtkWidget *widget, 
							 gpointer data);
static void sg_menu_layer_control			(GtkWidget *widget, 
							 gpointer data);

void
sg_plot_menu_show(GtkWidget *widget, GdkEventButton *event)
{
  GList *w;
  SGplot *plot = NULL;
  GtkWidget *data;
  GdkModifierType mods;

  w = plots;
  while(w){
    plot = (SGplot *)w->data;
    data = plot->real_canvas;
    if(data && data == widget) break;
    w = w->next;
  }

  active_plot = plot;

  gdk_window_get_pointer(widget->window, NULL, NULL, &mods);
  if(!(mods & GDK_BUTTON3_MASK)) return;

  gtk_menu_popup(GTK_MENU(plot_menu), NULL, NULL, NULL, NULL,
                 event->button, event->time);
}

void
sg_plot_build_menu()
{
  GtkWidget *item;
  GtkWidget *sub_menu, *sub_sub_menu;

  plot_menu = gtk_menu_new();

  /* FILE */

  sub_menu = gtk_menu_new();

  item=gtk_menu_item_new_with_label("Export");
  gtk_widget_show(item);
  gtk_menu_append(GTK_MENU(sub_menu),item);
  gtk_signal_connect(GTK_OBJECT(item),"activate",sg_menu_export, NULL);

  item=gtk_menu_item_new_with_label("Import");
  gtk_widget_show(item);
  gtk_menu_append(GTK_MENU(sub_menu),item);
  gtk_signal_connect(GTK_OBJECT(item),"activate",sg_menu_import, NULL);

  item=gtk_menu_item_new_with_label("File");
  gtk_widget_show(item);
  gtk_menu_append(GTK_MENU(plot_menu),item);

  gtk_menu_item_set_submenu(GTK_MENU_ITEM(item), sub_menu);

  /* LAYER */

  sub_sub_menu = gtk_menu_new();

  item=gtk_menu_item_new_with_label("2D");
  gtk_widget_show(item);
  gtk_menu_append(GTK_MENU(sub_sub_menu),item);
  gtk_signal_connect(GTK_OBJECT(item),"activate",sg_menu_new_layer, NULL);

  item=gtk_menu_item_new_with_label("3D");
  gtk_widget_show(item);
  gtk_menu_append(GTK_MENU(sub_sub_menu),item);
  gtk_signal_connect(GTK_OBJECT(item),"activate",sg_menu_new_layer_3d, NULL);

  item=gtk_menu_item_new_with_label("Polar");
  gtk_widget_show(item);
  gtk_menu_append(GTK_MENU(sub_sub_menu),item);
  gtk_signal_connect(GTK_OBJECT(item),"activate",sg_menu_new_layer_polar, NULL);

  sub_menu = gtk_menu_new();

  item=gtk_menu_item_new_with_label("New layer");
  gtk_widget_show(item);
  gtk_menu_append(GTK_MENU(sub_menu),item);
  gtk_menu_item_set_submenu(GTK_MENU_ITEM(item), sub_sub_menu);

  item=gtk_menu_item_new_with_label("Remove layer");
  gtk_widget_show(item);
  gtk_menu_append(GTK_MENU(sub_menu),item);
  gtk_signal_connect(GTK_OBJECT(item),"activate",sg_menu_remove_layer, NULL);

  item=gtk_menu_item_new_with_label("Layer control");
  gtk_widget_show(item);
  gtk_menu_append(GTK_MENU(sub_menu),item);
  gtk_signal_connect(GTK_OBJECT(item),"activate",sg_menu_layer_control, NULL);

  item=gtk_menu_item_new_with_label("Layer");
  gtk_widget_show(item);
  gtk_menu_append(GTK_MENU(plot_menu),item);

  gtk_menu_item_set_submenu(GTK_MENU_ITEM(item), sub_menu);

  /* PAGE */

  sub_menu = gtk_menu_new();

  item=gtk_menu_item_new_with_label("Fit page to window");
  gtk_widget_show(item);
  gtk_menu_append(GTK_MENU(sub_menu),item);
  gtk_signal_connect(GTK_OBJECT(item),"activate",sg_menu_fit_page, NULL);

  item=gtk_menu_item_new_with_label("Arrange all layers");
  gtk_widget_show(item);
  gtk_menu_append(GTK_MENU(sub_menu),item);
  gtk_signal_connect(GTK_OBJECT(item),"activate",sg_menu_arrange, NULL);

  item=gtk_menu_item_new_with_label("Page");
  gtk_widget_show(item);
  gtk_menu_append(GTK_MENU(plot_menu),item);

  gtk_menu_item_set_submenu(GTK_MENU_ITEM(item), sub_menu);

}

static void
sg_menu_fit_page(GtkWidget *widget, gpointer data)
{
  sg_plot_fit_page(active_plot);
}

static void
sg_menu_arrange(GtkWidget *widget, gpointer data)
{
  sg_arrange_dialog(GTK_PLOT_CANVAS(active_plot->real_canvas));
}

static void
sg_menu_new_layer(GtkWidget *widget, gpointer data)
{
  SGlayer *layer;

  layer = sg_layer_new(SG_PLOT_2D, .65, .45);
  sg_plot_add_layer(active_plot, layer, .175, .15);

  gtk_plot_canvas_paint(GTK_PLOT_CANVAS(active_plot->real_canvas));
  gtk_plot_canvas_refresh(GTK_PLOT_CANVAS(active_plot->real_canvas));
}

static void
sg_menu_new_layer_3d(GtkWidget *widget, gpointer data)
{
  SGlayer *layer;

  layer = sg_layer_new(SG_PLOT_3D, .65, .45);
  sg_plot_add_layer(active_plot, layer, .175, .15);

  gtk_plot_canvas_paint(GTK_PLOT_CANVAS(active_plot->real_canvas));
  gtk_plot_canvas_refresh(GTK_PLOT_CANVAS(active_plot->real_canvas));
}

static void
sg_menu_new_layer_polar(GtkWidget *widget, gpointer data)
{
  SGlayer *layer;

  layer = sg_layer_new(SG_PLOT_POLAR, .75, .45);
  sg_plot_add_layer(active_plot, layer, .175, .15);

  gtk_plot_canvas_paint(GTK_PLOT_CANVAS(active_plot->real_canvas));
  gtk_plot_canvas_refresh(GTK_PLOT_CANVAS(active_plot->real_canvas));
}

static void
sg_menu_remove_layer(GtkWidget *widget, gpointer data)
{
  SGlayer *layer;

  layer = active_plot->active_layer;

  if(active_plot->nlayers > 1 && sg_accept_dialog("Remove the selected layer?", 0))
       sg_plot_remove_layer(active_plot, layer);

  sg_plot_set_active_layer(active_plot, (SGlayer *)active_plot->layers->data); 
}

static void
sg_menu_layer_control(GtkWidget *widget, gpointer data)
{
  sg_layer_control(NULL);
}

static void
sg_menu_import(GtkWidget *widget, gpointer data)
{
/*
  sg_message_dialog("Sorry, this feature is not implemented", 2);
*/
  sg_plot_import();
}

static void
sg_menu_export(GtkWidget *widget, gpointer data)
{
  sg_plot_export();
}

