/*  SciGraphica - Scientific graphics and data manipulation
 *  Copyright (C) 2000 Adrian E. Feiguin <feiguin@ifir.edu.ar>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include <math.h>
#include <stdio.h>
#include <gdk/gdk.h>
#include <gtk/gtk.h>
#include <gtkextra/gtkextra.h>
#include "main.h"
#include "../pixmaps/new.xpm"
#include "../pixmaps/open.xpm"
#include "../pixmaps/save.xpm"
#include "../pixmaps/save_as.xpm"
#include "../pixmaps/new_sheet.xpm"
#include "../pixmaps/new_plot.xpm"
#include "../pixmaps/new_function.xpm"
#include "../pixmaps/plot_small.xpm"
#include "../pixmaps/sheet_small.xpm"
#include "../pixmaps/function_small.xpm"
#include "../pixmaps/new_2d.xpm"
#include "../pixmaps/new_3d.xpm"
#include "../pixmaps/new_polar.xpm"
#include "../pixmaps/new_2d_small.xpm"
#include "../pixmaps/new_3d_small.xpm"
#include "../pixmaps/new_polar_small.xpm"
#include "python/python_term.h"

static char *popup_items[]={
                "Open",
                "Rename",
                "Remove",
};

static void sg_menu_exit				(GtkWidget *widget, 
							 gpointer data);
static void sg_menu_new_project				(GtkWidget *widget, 
							 gpointer data);
static void sg_menu_new_plot				(GtkWidget *widget, 
							 gpointer data);
static void sg_menu_new_plot_3d				(GtkWidget *widget, 
							 gpointer data);
static void sg_menu_new_plot_polar			(GtkWidget *widget, 
							 gpointer data);
static void sg_menu_new_worksheet			(GtkWidget *widget, 
							 gpointer data);
static void sg_menu_new_function                        (GtkWidget *widget,
                                                         gpointer data);
static void sg_menu_open				(GtkWidget *widget, 
							 gpointer data);
static void sg_menu_save				(GtkWidget *widget, 
							 gpointer data);
static void sg_menu_save_as				(GtkWidget *widget, 
							 gpointer data);

void
sg_worksheet_popup_show(GtkIconListItem *item, GdkEventButton *event)
{
    active_icon = item;

    gtk_menu_popup(GTK_MENU(worksheet_popup), NULL, NULL, NULL, NULL,
                   event->button, event->time);
}

void
sg_plot_popup_show(GtkIconListItem *item, GdkEventButton *event)
{
    active_icon = item;

    gtk_menu_popup(GTK_MENU(plot_popup), NULL, NULL, NULL, NULL,
                   event->button, event->time);
}


static void
plot_popup_activated(GtkWidget *widget, gpointer data)
{
 gchar *item;
 GtkIconList *iconlist;
 SGplot *plot;

 item = (gchar *)data;
 iconlist = GTK_ICON_LIST(iconlist2);

 plot =  (SGplot *) gtk_icon_list_get_link(active_icon);

 if(strcmp(item,"Open")==0)
   sg_plot_open(plot);

 if(strcmp(item,"Remove")==0){
   gint veto;
   gchar text[100]; 
   
   sprintf(text,"Remove %s ?", plot->name);

   veto = sg_accept_dialog(text, 1);
   if(!veto) return;

   sg_project_remove_plot(plot);
 }

 if(strcmp(item,"Rename")==0)
   gtk_icon_list_set_active_icon(iconlist, active_icon);

}

static void
worksheet_popup_activated(GtkWidget *widget, gpointer data)
{
 gchar *item;
 GtkIconList *iconlist;
 SGworksheet *worksheet;

 item = (gchar *)data;
 iconlist = GTK_ICON_LIST(iconlist1);

 worksheet = (SGworksheet *) gtk_icon_list_get_link(active_icon);

 if(strcmp(item,"Open")==0)
   sg_worksheet_open(worksheet);

 if(strcmp(item,"Remove")==0){
   gint veto;
   gchar text[100];

   sprintf(text,"Remove %s ?", worksheet->name);

   veto = sg_accept_dialog(text, 1);
   if(!veto) return;

   sg_project_remove_worksheet(worksheet);
 }

 if(strcmp(item,"Rename")==0)
   gtk_icon_list_set_active_icon(iconlist, active_icon);

}


void
sg_project_build_popups()
{
        GtkWidget *item;
        int i;

        worksheet_popup = gtk_menu_new();

        for (i=0; i < (sizeof(popup_items)/sizeof(popup_items[0])) ; i++){
                item=gtk_menu_item_new_with_label(popup_items[i]);
                gtk_signal_connect(GTK_OBJECT(item),"activate",
                                   (GtkSignalFunc) worksheet_popup_activated,
                                   popup_items[i]);
                gtk_widget_show(item);
                gtk_menu_append(GTK_MENU(worksheet_popup),item);
        }

        plot_popup = gtk_menu_new();

        for (i=0; i < (sizeof(popup_items)/sizeof(popup_items[0])) ; i++){
                item=gtk_menu_item_new_with_label(popup_items[i]);
                gtk_signal_connect(GTK_OBJECT(item),"activate",
                                   (GtkSignalFunc) plot_popup_activated,
                                   popup_items[i]);
                gtk_widget_show(item);
                gtk_menu_append(GTK_MENU(plot_popup),item);
        }

}

#ifdef WITH_GNOME

static GnomeUIInfo new_menu[] = {
  {GNOME_APP_UI_ITEM, N_("P_roject"),
   N_("Create new project"),
   sg_menu_new_project, NULL,
   NULL, GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_MENU_NEW, 'r', GDK_CONTROL_MASK},

  {GNOME_APP_UI_ITEM, N_("_Worksheet"),
   N_("Create new worksheet"),
   sg_menu_new_worksheet, NULL,
   NULL, GNOME_APP_PIXMAP_DATA, sheet_small_xpm, 'w', GDK_CONTROL_MASK},

  {GNOME_APP_UI_ITEM, N_("2D _Plot"),
   N_("Create new plot"),
   sg_menu_new_plot, NULL,
   NULL, GNOME_APP_PIXMAP_DATA, new_2d_small_xpm, 'p', GDK_CONTROL_MASK},

  {GNOME_APP_UI_ITEM, N_("3_D Plot"),
   N_("Create new 3D plot"),
   sg_menu_new_plot_3d, NULL,
   NULL, GNOME_APP_PIXMAP_DATA, new_3d_small_xpm, 'd', GDK_CONTROL_MASK},

  {GNOME_APP_UI_ITEM, N_("Po_lar Plot"),
   N_("Create new polar plot"),
   sg_menu_new_plot_polar, NULL,
   NULL, GNOME_APP_PIXMAP_DATA, new_polar_small_xpm, 'l', GDK_CONTROL_MASK},

  {GNOME_APP_UI_ITEM, N_("_Function"),
   N_("Create new function"),
   sg_menu_new_function, NULL,
   NULL, GNOME_APP_PIXMAP_DATA, function_small_xpm, 'f', GDK_CONTROL_MASK},

   GNOMEUIINFO_END
 };

static GnomeUIInfo gnome_file_menu[] = {
   GNOMEUIINFO_SUBTREE_STOCK(N_("_New"), new_menu,GNOME_STOCK_MENU_NEW),
   GNOMEUIINFO_MENU_OPEN_ITEM(sg_menu_open,NULL),
   GNOMEUIINFO_MENU_SAVE_ITEM(sg_menu_save,NULL),
   GNOMEUIINFO_MENU_SAVE_AS_ITEM(sg_menu_save_as,NULL),
   GNOMEUIINFO_SEPARATOR,
   GNOMEUIINFO_MENU_EXIT_ITEM(sg_menu_exit,NULL),
   GNOMEUIINFO_END
 };

static GnomeUIInfo gnome_help_menu[] = {
   GNOMEUIINFO_MENU_ABOUT_ITEM(sg_about_dialog,NULL),
   GNOMEUIINFO_END
 };

static GnomeUIInfo gnome_win_menu[] = {
   GNOMEUIINFO_ITEM_STOCK( "Python _terminal", "Open Python terminal window", create_python_term,
    GNOME_STOCK_MENU_ATTACH),
   GNOMEUIINFO_END
 };

static GnomeUIInfo gnome_main_menu[] = {
   GNOMEUIINFO_MENU_FILE_TREE(gnome_file_menu),
   GNOMEUIINFO_SUBTREE(N_("_Window"), gnome_win_menu),
   GNOMEUIINFO_SUBTREE(N_("_Help"), gnome_help_menu),

   GNOMEUIINFO_END
 };

void
sg_project_build_gnome_menu(GnomeApp *app)
{
  gnome_app_create_menus(app, gnome_main_menu);
}

GtkWidget *
sg_project_build_toolbar()
{
  GnomeUIInfo main_toolbar[] =
  {
    GNOMEUIINFO_ITEM_STOCK("New", "Create new project", sg_menu_new_project,
                            GNOME_STOCK_PIXMAP_NEW),
    GNOMEUIINFO_ITEM_STOCK("Open", "Open project", sg_menu_open,
                            GNOME_STOCK_PIXMAP_OPEN),
    GNOMEUIINFO_ITEM_STOCK("Save", "Save project", sg_menu_save,
                            GNOME_STOCK_PIXMAP_SAVE),
    GNOMEUIINFO_SEPARATOR,

    GNOMEUIINFO_ITEM("Sheet", "Create new worksheet", sg_menu_new_worksheet, 
                      new_sheet_xpm),
    GNOMEUIINFO_ITEM("2D", "Create new plot", sg_menu_new_plot, 
                      new_2d_xpm),
    GNOMEUIINFO_ITEM("3D", "Create new 3D plot", sg_menu_new_plot_3d,
                      new_3d_xpm),
    GNOMEUIINFO_ITEM("Polar", "Create new polar plot", sg_menu_new_plot_polar, 
                      new_polar_xpm),
    GNOMEUIINFO_ITEM("Function", "Create new function", sg_menu_new_function, 
                      new_function_xpm),
    GNOMEUIINFO_END
  };
  gnome_app_create_toolbar(GNOME_APP(main_window), main_toolbar);
  return NULL;
}


#else

GtkWidget *
sg_project_build_menu()
{
 GtkWidget *root_menu;
 GtkWidget *menu;
 GtkWidget *menu_bar;
 GtkWidget *menu_item;
 GtkWidget *file_menu;

 menu_bar = gtk_menu_bar_new();

 file_menu = gtk_menu_new();

 menu_item = gtk_menu_item_new_with_label("Project");
 gtk_menu_append (GTK_MENU(file_menu), menu_item);
 gtk_widget_show(menu_item);
 gtk_signal_connect(GTK_OBJECT(menu_item),"activate",sg_menu_new_project, NULL);

 menu_item = gtk_menu_item_new_with_label("Worksheet");
 gtk_menu_append (GTK_MENU(file_menu), menu_item);
 gtk_widget_show(menu_item);
 gtk_signal_connect(GTK_OBJECT(menu_item),"activate",sg_menu_new_worksheet, NULL);

 menu_item = gtk_menu_item_new_with_label("2D Plot");
 gtk_menu_append (GTK_MENU(file_menu), menu_item);
 gtk_widget_show(menu_item);
 gtk_signal_connect(GTK_OBJECT(menu_item),"activate",sg_menu_new_plot, NULL);

 menu_item = gtk_menu_item_new_with_label("3D Plot");
 gtk_menu_append (GTK_MENU(file_menu), menu_item);
 gtk_widget_show(menu_item);
 gtk_signal_connect(GTK_OBJECT(menu_item),"activate",sg_menu_new_plot_3d, NULL);

 menu_item = gtk_menu_item_new_with_label("Polar Plot");
 gtk_menu_append (GTK_MENU(file_menu), menu_item);
 gtk_widget_show(menu_item);
 gtk_signal_connect(GTK_OBJECT(menu_item),"activate",sg_menu_new_plot_polar, NULL);

 menu_item = gtk_menu_item_new_with_label("Function");
 gtk_menu_append (GTK_MENU(file_menu), menu_item);
 gtk_widget_show(menu_item);
 gtk_signal_connect(GTK_OBJECT(menu_item),"activate",sg_menu_new_function, NULL);

 menu = gtk_menu_new();

 menu_item = gtk_menu_item_new_with_label("New");
 gtk_menu_append (GTK_MENU(menu), menu_item);
 gtk_widget_show(menu_item);

 gtk_menu_item_set_submenu(GTK_MENU_ITEM(menu_item),file_menu);

 menu_item = gtk_menu_item_new_with_label("Open");
 gtk_menu_append (GTK_MENU(menu), menu_item);
 gtk_signal_connect(GTK_OBJECT(menu_item),"activate",sg_menu_open, NULL);
 gtk_widget_show(menu_item);

 menu_item = gtk_menu_item_new_with_label("Save");
 gtk_menu_append (GTK_MENU(menu), menu_item);
 gtk_signal_connect(GTK_OBJECT(menu_item),"activate",sg_menu_save, NULL);
 gtk_widget_show(menu_item);

 menu_item = gtk_menu_item_new_with_label("Save As");
 gtk_menu_append (GTK_MENU(menu), menu_item);
 gtk_signal_connect(GTK_OBJECT(menu_item),"activate",sg_menu_save_as, NULL);
 gtk_widget_show(menu_item);

 menu_item = gtk_menu_item_new_with_label("Exit");
 gtk_menu_append (GTK_MENU(menu), menu_item);
 gtk_widget_show(menu_item);
 gtk_signal_connect(GTK_OBJECT(menu_item),"activate",sg_menu_exit, NULL);
 
 root_menu = gtk_menu_item_new_with_label("File");
 gtk_menu_item_set_submenu (GTK_MENU_ITEM(root_menu), menu);
 gtk_menu_bar_append(GTK_MENU_BAR(menu_bar), root_menu);
 gtk_widget_show(root_menu);


 menu = gtk_menu_new();

 menu_item = gtk_menu_item_new_with_label("Python terminal");
 gtk_menu_append (GTK_MENU(menu), menu_item);
 gtk_widget_show(menu_item);
 gtk_signal_connect(GTK_OBJECT(menu_item),"activate", 
                    GTK_SIGNAL_FUNC(create_python_term), NULL);
 
 root_menu = gtk_menu_item_new_with_label("Window");
 gtk_menu_item_set_submenu (GTK_MENU_ITEM(root_menu), menu);
 gtk_menu_bar_append(GTK_MENU_BAR(menu_bar), root_menu);
 gtk_widget_show(root_menu);


 menu = gtk_menu_new();

/*
 menu_item = gtk_menu_item_new_with_label("Contents");
 gtk_menu_append (GTK_MENU(menu), menu_item);
 gtk_widget_show(menu_item);
*/

 menu_item = gtk_menu_item_new_with_label("About SciGraphica");
 gtk_menu_append (GTK_MENU(menu), menu_item);
 gtk_widget_show(menu_item);
 gtk_signal_connect(GTK_OBJECT(menu_item),"activate",sg_about_dialog, NULL);

 root_menu = gtk_menu_item_new_with_label("Help");
 gtk_menu_item_right_justify(GTK_MENU_ITEM(root_menu));
 gtk_menu_item_set_submenu (GTK_MENU_ITEM(root_menu), menu);
 gtk_menu_bar_append(GTK_MENU_BAR(menu_bar), root_menu);
 gtk_widget_show(root_menu);

 return menu_bar;
}


GtkWidget *
sg_project_build_toolbar()
{
  GtkWidget *pixmap;
  GtkWidget *button;
  GtkWidget *toolbar;
  GdkPixmap *tool;
  GdkBitmap *mask;
  GdkColormap *colormap;

  colormap = gdk_colormap_get_system();

  toolbar = gtk_toolbar_new(GTK_ORIENTATION_HORIZONTAL, GTK_TOOLBAR_CHILD_BUTTON);
  gtk_container_set_border_width(GTK_CONTAINER(toolbar), 2);
  gtk_toolbar_set_button_relief(GTK_TOOLBAR(toolbar), GTK_RELIEF_NONE);

  tool=gdk_pixmap_colormap_create_from_xpm_d(NULL, colormap, &mask, NULL,
                                             new_xpm);
  pixmap = gtk_pixmap_new(tool, mask);
  button = gtk_toolbar_append_item(GTK_TOOLBAR(toolbar),
	  		           NULL,
			           "Start a new project",
			           "New project",
			           pixmap,
			           sg_menu_new_project,
			           NULL);

  tool=gdk_pixmap_colormap_create_from_xpm_d(NULL, colormap, &mask, NULL,
                                             open_xpm);
  pixmap = gtk_pixmap_new(tool, mask);
  button = gtk_toolbar_append_item(GTK_TOOLBAR(toolbar),
			           NULL,
			           "Open a project",
			           "Open project",
			           pixmap,
			           sg_menu_open,
			           NULL);


  tool=gdk_pixmap_colormap_create_from_xpm_d(NULL, colormap, &mask, NULL,
                                             save_xpm);
  pixmap = gtk_pixmap_new(tool, mask);
  button = gtk_toolbar_append_item(GTK_TOOLBAR(toolbar),
			           NULL,
			           "Save project",
			           "Save project",
			           pixmap,
			           sg_menu_save,
			           NULL);

  gtk_toolbar_set_space_style(GTK_TOOLBAR(toolbar), GTK_TOOLBAR_SPACE_LINE);
  gtk_toolbar_append_space(GTK_TOOLBAR(toolbar));

  tool=gdk_pixmap_colormap_create_from_xpm_d(NULL, colormap, &mask, NULL,
                                             new_sheet_xpm);
  pixmap = gtk_pixmap_new(tool, mask);
  button = gtk_toolbar_append_item(GTK_TOOLBAR(toolbar),
			           NULL,
			           "Create a new worksheet",
			           "Create worksheet",
			           pixmap,
			           sg_menu_new_worksheet,
			           NULL);

  tool=gdk_pixmap_colormap_create_from_xpm_d(NULL, colormap, &mask, NULL,
                                             new_2d_xpm);
  pixmap = gtk_pixmap_new(tool, mask);
  button = gtk_toolbar_append_item(GTK_TOOLBAR(toolbar),
			           NULL,
			           "Create a new Plot",
			           "Create plot",
			           pixmap,
			           sg_menu_new_plot,
			           NULL);

  tool=gdk_pixmap_colormap_create_from_xpm_d(NULL, colormap, &mask, NULL,
                                             new_3d_xpm);
  pixmap = gtk_pixmap_new(tool, mask);
  button = gtk_toolbar_append_item(GTK_TOOLBAR(toolbar),
			           NULL,
			           "Create a new 3D Plot",
			           "Create 3D plot",
			           pixmap,
			           sg_menu_new_plot_3d,
			           NULL);

  tool=gdk_pixmap_colormap_create_from_xpm_d(NULL, colormap, &mask, NULL,
                                             new_polar_xpm);
  pixmap = gtk_pixmap_new(tool, mask);
  button = gtk_toolbar_append_item(GTK_TOOLBAR(toolbar),
			           NULL,
			           "Create a new Polar Plot",
			           "Create Polar plot",
			           pixmap,
			           sg_menu_new_plot_polar,
			           NULL);

  tool=gdk_pixmap_colormap_create_from_xpm_d(NULL, colormap, &mask, NULL,
                                             new_function_xpm);
  pixmap = gtk_pixmap_new(tool, mask);
  button = gtk_toolbar_append_item(GTK_TOOLBAR(toolbar),
			           NULL,
			           "Create a new Function",
			           "Create function",
			           pixmap,
			           sg_menu_new_function,
			           NULL);
  return(toolbar);
}

#endif

static void
sg_menu_exit(GtkWidget *widget, gpointer data)
{
  gint veto;
  gchar label[255];

  sg_dialogs_destroy();

  sprintf(label, "Save project \"%s\" before leaving?", last_project_filename);
  veto = sg_accept_dialog(label, 0);
  switch(veto) {
    case YES_CLICKED:
      sg_menu_save(widget, data);
    case NO_CLICKED:
      break;
    case CANCEL_CLICKED:
      return;
  }

  main_quit();
}

static void
sg_menu_new_plot(GtkWidget *widget, gpointer data)
{
  sg_project_new_plot_with_layer(SG_PLOT_2D);
}

static void
sg_menu_new_plot_3d(GtkWidget *widget, gpointer data)
{
  sg_project_new_plot_with_layer(SG_PLOT_3D);
}

static void
sg_menu_new_plot_polar(GtkWidget *widget, gpointer data)
{
  sg_project_new_plot_with_layer(SG_PLOT_POLAR);
}

static void
sg_menu_new_worksheet(GtkWidget *widget, gpointer data)
{
  sg_project_new_worksheet();
}

static void
sg_menu_new_function(GtkWidget *widget, gpointer data)
{
  sg_function_dialog_new();
}

static void
sg_menu_new_project(GtkWidget *widget, gpointer data)
{
  gint veto;
  gchar label[80];

  sprintf(label, "Save project \"%s\" ?", last_project_filename);
  veto = sg_accept_dialog(label, 0);
  switch (veto){
    case YES_CLICKED:
       sg_menu_save(widget, data);
       break;
    case NO_CLICKED:
       break;
    case CANCEL_CLICKED:
       return;
  }

  sg_project_close();
  sg_project_new();
  sg_project_new_worksheet();
}

static void
sg_menu_open(GtkWidget *widget, gpointer data)
{
  gboolean veto;
  gchar label[80];

  sprintf(label, "Save project \"%s\" ?", last_project_filename);
  veto = sg_accept_dialog(label, 0);
  switch (veto){
    case YES_CLICKED:
       sg_menu_save(widget, data);
       break;
    case NO_CLICKED:
       break;
    case CANCEL_CLICKED:
       return;
  }

  sg_project_close();
  sg_project_new();
  sg_project_import();
}

static void
sg_menu_save(GtkWidget *widget, gpointer data)
{
  sg_project_export();
}

static void
sg_menu_save_as(GtkWidget *widget, gpointer data)
{
  sg_project_export_as();
}
