/*  SciGraphica - Scientific graphics and data manipulation
 *  Copyright (C) 2000 Adrian E. Feiguin <feiguin@ifir.edu.ar>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include <gdk/gdk.h>
#include <gtk/gtk.h>
#include <gtkextra/gtkextra.h>
#include "main.h"
#include "sg_worksheet.h"
#include "sg_worksheet_tools.h"

#include "../pixmaps/open.xpm"
#include "../pixmaps/save.xpm"
#include "../pixmaps/cut.xpm"
#include "../pixmaps/copy.xpm"
#include "../pixmaps/paste.xpm"
#include "../pixmaps/add_column.xpm"
#include "../pixmaps/reload.xpm"

static void worksheet_import		(GtkWidget *widget, gpointer data);
static void worksheet_export		(GtkWidget *widget, gpointer data);
static void worksheet_add_column 	(GtkWidget *widget, gpointer data);
static void worksheet_cut	 	(GtkWidget *widget, gpointer data);
static void worksheet_copy	 	(GtkWidget *widget, gpointer data);
static void worksheet_paste	 	(GtkWidget *widget, gpointer data);
static void worksheet_recalc	 	(GtkWidget *widget, gpointer data);

#ifdef WITH_GNOME
GtkWidget *
sg_worksheet_build_toolbar(SGworksheet *worksheet)
{
  GnomeUIInfo worksheet_toolbar[] =
  {
      GNOMEUIINFO_ITEM_DATA("Open", "Open file", worksheet_import,
                             worksheet,
                             open_xpm),
      GNOMEUIINFO_ITEM_DATA("Save", "Save worksheet", worksheet_export,
                             worksheet,
                             save_xpm),
      GNOMEUIINFO_SEPARATOR,
      GNOMEUIINFO_ITEM_DATA("Cut", "Cut", worksheet_cut,
                             worksheet,
                             cut_xpm),
      GNOMEUIINFO_ITEM_DATA("Copy", "Copy", worksheet_copy,
                             worksheet,
                             copy_xpm),
      GNOMEUIINFO_ITEM_DATA("Paste", "Paste", worksheet_paste,
                             worksheet,
                             paste_xpm),
      GNOMEUIINFO_SEPARATOR,
      GNOMEUIINFO_ITEM_DATA("Column", "Add Column", worksheet_add_column,
                             worksheet,
                            add_column_xpm),
      GNOMEUIINFO_ITEM_DATA("Recalc", "Recalculate selected cells", worksheet_recalc,
                             worksheet,
                             reload_xpm),

      GNOMEUIINFO_END
  };

  gnome_app_create_toolbar(GNOME_APP(worksheet->window) , worksheet_toolbar);
  return NULL;
}

#else

GtkWidget *
sg_worksheet_build_toolbar(SGworksheet *worksheet)
{
  GtkWidget *pixmap;
  GtkWidget *button;
  GtkWidget *toolbar;
  GdkPixmap *tool;
  GdkBitmap *mask;
  GdkColormap *colormap;

  colormap = gdk_colormap_get_system();

  toolbar = gtk_toolbar_new(GTK_ORIENTATION_HORIZONTAL, GTK_TOOLBAR_CHILD_BUTTON);
  gtk_container_set_border_width(GTK_CONTAINER(toolbar), 2);
  gtk_toolbar_set_button_relief(GTK_TOOLBAR(toolbar), GTK_RELIEF_NONE);

  tool=gdk_pixmap_colormap_create_from_xpm_d(NULL, colormap, &mask, NULL,
                                             open_xpm);
  pixmap = gtk_pixmap_new(tool, mask);
  button = gtk_toolbar_append_item(GTK_TOOLBAR(toolbar),
                                   NULL,
                                   "Open",
                                   "Open file",
                                   pixmap,
                                   worksheet_import,
                                   worksheet);


  tool=gdk_pixmap_colormap_create_from_xpm_d(NULL, colormap, &mask, NULL,
                                             save_xpm);
  pixmap = gtk_pixmap_new(tool, mask);
  button = gtk_toolbar_append_item(GTK_TOOLBAR(toolbar),
                                   NULL,
                                   "Save",
                                   "Save worksheet",
                                   pixmap,
                                   worksheet_export,
                                   worksheet);

  gtk_toolbar_set_space_style(GTK_TOOLBAR(toolbar), GTK_TOOLBAR_SPACE_LINE);
  gtk_toolbar_append_space(GTK_TOOLBAR(toolbar));


  tool=gdk_pixmap_colormap_create_from_xpm_d(NULL, colormap, &mask, NULL,
                                             cut_xpm);
  pixmap = gtk_pixmap_new(tool, mask);
  button = gtk_toolbar_append_item(GTK_TOOLBAR(toolbar),
                                   NULL,
                                   "Cut",
                                   "Cut",
                                   pixmap,
                                   worksheet_cut,
                                   worksheet);

  tool=gdk_pixmap_colormap_create_from_xpm_d(NULL, colormap, &mask, NULL,
                                             copy_xpm);
  pixmap = gtk_pixmap_new(tool, mask);
  button = gtk_toolbar_append_item(GTK_TOOLBAR(toolbar),
                                   NULL,
                                   "Copy",
                                   "Copy",
                                   pixmap,
                                   worksheet_copy,
                                   worksheet);

  tool=gdk_pixmap_colormap_create_from_xpm_d(NULL, colormap, &mask, NULL,
                                             paste_xpm);
  pixmap = gtk_pixmap_new(tool, mask);
  button = gtk_toolbar_append_item(GTK_TOOLBAR(toolbar),
                                   NULL,
                                   "Paste",
                                   "Paste",
                                   pixmap,
                                   worksheet_paste,
                                   worksheet);

  gtk_toolbar_append_space(GTK_TOOLBAR(toolbar));

  tool=gdk_pixmap_colormap_create_from_xpm_d(NULL, colormap, &mask, NULL,
                                             add_column_xpm);
  pixmap = gtk_pixmap_new(tool, mask);
  button = gtk_toolbar_append_item(GTK_TOOLBAR(toolbar),
                                   NULL,
                                   "Column",
                                   "Add Column",
                                   pixmap,
                                   worksheet_add_column,
                                   worksheet);

  tool=gdk_pixmap_colormap_create_from_xpm_d(NULL, colormap, &mask, NULL,
                                             reload_xpm);
  pixmap = gtk_pixmap_new(tool, mask);
  button = gtk_toolbar_append_item(GTK_TOOLBAR(toolbar),
                                   NULL,
                                   "Recalc",
                                   "Recalculate selected cells",
                                   pixmap,
                                   worksheet_recalc,
                                   worksheet);

  return(toolbar);
}

#endif

static void
worksheet_import (GtkWidget *widget, gpointer data)
{
  active_worksheet = (SGworksheet *) data;
  sg_worksheet_import();
}

static void
worksheet_export (GtkWidget *widget, gpointer data)
{
  active_worksheet = (SGworksheet *) data;
  sg_worksheet_export();
}

static void
worksheet_add_column (GtkWidget *widget, gpointer data)
{
  active_worksheet = (SGworksheet *) data;
  sg_worksheet_add_columns(active_worksheet, 1);
}

static void
worksheet_cut (GtkWidget *widget, gpointer data)
{
  active_worksheet = (SGworksheet *) data;
  sg_clipboard_copy(active_worksheet,TRUE);
}

static void
worksheet_copy (GtkWidget *widget, gpointer data)
{
  active_worksheet = (SGworksheet *) data;
  sg_clipboard_copy(active_worksheet,FALSE);
}

static void
worksheet_paste (GtkWidget *widget, gpointer data)
{
  active_worksheet = (SGworksheet *) data;
  sg_clipboard_paste(active_worksheet);
}

static void
worksheet_recalc (GtkWidget *widget, gpointer data)
{ GtkSheet *sheet;
  active_worksheet = (SGworksheet *) data;
  sheet=GTK_SHEET(active_worksheet->sheet);
  
  sg_worksheet_unupdate_exp_range(active_worksheet,
               sheet->range.row0,sheet->range.rowi,
               sheet->range.col0,sheet->range.coli);

  sg_worksheet_update_exp_range(active_worksheet,
               sheet->range.row0,sheet->range.rowi,
               sheet->range.col0,sheet->range.coli);
}
